---
layout: "default"
isPage: true
---

# Tutorial

- [Install ESDoc](#install-esdoc)
- [Create Config File](#create-config-file)
- [Write Documentation Tags](#write-documentation-tags)
- [Document Coverage](#document-coverage)
- [Integrate Test Codes](#integrate-test-codes)
- [Integrate Manual](#integrate-manual)
- [Appendix](#appendix)
  - [Target Code Style](#target-code-style)
  - [Target Identifier](#target-identifier)

This tutorial explains how to generate a document with the following JavaScript source code.

```
my-project/
├── README.md
├── package.json
└── src
    ├── MyClass.js
    └── MySuperClass.js
```

<p class="file-path">my-project/src/MyClass.js</p>

```javascript
import MySuperClass from './MySuperClass.js';

export default class MyClass extends MySuperClass {
  constructor(name = 'anonymous') {
    super();
    this._name = name;
  }

  sayMyName() {
    return `My name is ${this._name}`;
  }
}
```

<p class="file-path">my-project/src/MySuperClass.js</p>

```javascript
export default class MySuperClass {
  constructor() {
  }

  sayHello(){
    return 'Hello!';
  }
}
```

<p class="file-path">my-project/README.md</p>

```markdown
# My Project
this is My Project README
```



## Install ESDoc
Install ESDoc with npm.

```
npm install esdoc -g
esdoc -h
```



## Create Config File
Create ESDoc config file.

<p class="file-path">my-project/esdoc.json</p>

```json
{
  "source": "./src",
  "destination": "./out/esdoc"
}
```

Execute ESDoc and see output document.
```
esdoc -c ./esdoc.json
open ./out/esdoc/index.html
```

<img class="screen-shot" src="./image/tutorial/create-config-file1.png">

This is minimum configuration.
You can see full configuration in the [Config](./config.html) page.




## Write Documentation Tags
The documentation is not good, because it is generated by the source code without tag(e.g. ``@param``).
So, write documentation tags.

<p class="file-path">my-project/src/MyClass.js</p>

```javascript
import MySuperClass from './MySuperClass.js';

/**
 * this is MyClass description.
 */
export default class MyClass extends MySuperClass {
  /**
   * this is MyClass constructor description.
   * @param {string} [name="anonymous"] - this is name description.
   */
  constructor(name = 'anonymous') {
    super();
    this._name = name;
  }

  /**
   * this is sayMyName description
   * @returns {string} this is return description.
   */
  sayMyName() {
    return `My name is ${this._name}`;
  }
}
```

<p class="file-path">my-project/src/SuperMyClass.js</p>

```javascript
/**
 * this is MySuperClass description.
 */
export default class MySuperClass {
  /**
   * this is MySuperClass constructor description.
   */
  constructor() {
  }

  /**
   * this is sayHello description.
   * @returns {string} this is return description.
   */
  sayHello(){
    return 'Hello!';
  }
}
```

execute ESDoc and see output document.
```
esdoc -c ./esdoc.json
open ./out/esdoc/index.html
```

<img class="screen-shot" src="./image/tutorial/write-document-tag1.png">

Congratulation! The document is good!

ESDoc has some tags. [Tags](./tags.html) page describes full tags.
You can see all tags in the [Tags](./tags.html) page.




## Document Coverage
ESDoc measures document coverage.

See ``my-project/out/esdoc/source.html``

<img class="screen-shot" src="./image/tutorial/document-coverage1.png">

If you want to display documentation coverage badge, use badge of [ESDoc Hosting Service](https://doc.esdoc.org/-/faq.html).




## Integrate Test Codes
ESDoc can integrate test codes into documentation.
(Now, support only [Mocha](http://mochajs.org/))

Write test code.

<p class="file-path">my-project/test/MyClassTest.js</p>

```javascript
import assert from 'assert';
import MyClass from '../src/MyClass.js';

describe('MyClass is super useful class.', ()=>{
  it('say my name', ()=>{
    let foo = new MyClass('Alice');
    assert.equal(foo.sayMyName(), 'My name is Alice');
  })
});
```

And add test configuration.

<p class="file-path">my-project/esdoc.json</p>

```json
{
  "source": "./src",
  "destination": "./out/esdoc",
  "test": {
    "type": "mocha",
    "source": "./test"
  }
}
```

Execute ESDoc and see output document.

```
esdoc -c ./esdoc.json
open ./out/esdoc/index.html
```

<img class="screen-shot" src="./image/tutorial/generate-document-from-test-code1.png">

Write tags for test code

<p class="file-path">my-project/test/MyClassTest.js</p>

```javascript
import assert from 'assert';
import MyClass from '../src/MyClass.js';

/** @test {MyClass} */
describe('MyClass is super useful class.', ()=>{

  /** @test {MyClass#sayMyName} */
  it('say my name', ()=>{
    let foo = new MyClass('Alice');
    assert.equal(foo.sayMyName(), 'My name is Alice');
  })
});
```

<img class="screen-shot" src="./image/tutorial/generate-document-from-test-code2.png">

<img class="screen-shot" src="./image/tutorial/generate-document-from-test-code3.png">

## Integrate Manual
ESDoc can integrate manual into documentation.
Write manual that is overview, installation, usage, example, FAQ and changelog.

<p class="file-path">my-project/manual/overview.md</p>

```markdown
# Overview
This is my project overview.
<img src="./asset/overview.png">

## Author
Alice

## License
MIT
```

And add manual configuration.

```json
{
  "source": "./src",
  "destination": "./out/esdoc",
  "test": {
    "type": "mocha",
    "source": "./test"
  },
  "manual": {
    "asset": "./manual/asset",
    "overview": ["./manual/overview.md"],
    "installation": ["./manual/installation.md"],
    "usage": ["./manual/usage.md"],
    "tutorial": ["./manual/tutorial.md"],
    "configuration": ["./manual/configuration.md"],
    "example": ["./manual/example.md"],
    "faq": ["./manual/faq.md"],
    "changelog": ["./CHANGELOG.md"]
  }
}
```

You can specify a only part of these manuals (e.g. only overview and installation).

Execute ESDoc and see output document.

```
esdoc -c ./esdoc.json
open ./out/esdoc/index.html
```

<img class="screen-shot" src="./image/tutorial/manual1.png">

<img class="screen-shot" src="./image/tutorial/manual2.png">

----

## Appendix

### Target Code Style
ESDoc targets at ES6 ``class`` and ``import/export`` style.
- Use class, extends, static, constructor, method, get and set syntax.
  - If you want to target at ES5, might better to use JSDoc.
- Use import and export syntax for module.

### Target Identifier
ESDoc processes only top level identifier.

```javascript
// this is processed.
class MyClass {
}

(function(){
  // this is not processed.
  class MyClass{
  }
})();
```

You can control target identifiers with ``access``, ``unexportIdentifier`` and ``undocumentIdentifier`` in config file.
See [Config](./config.html).

----

Tutorial is finish! Please enjoy writing documentation with ESDoc!
